package com.ruoyi.system.user.service;

import java.math.BigDecimal;
import java.util.List;

import com.ruoyi.bst.balanceLog.domain.enums.BalanceLogBstType;
import com.ruoyi.bst.changeRecord.domain.enums.ChangeRecordBstType;
import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.system.user.domain.UserQuery;
import com.ruoyi.system.user.domain.vo.UserConsumeVO;
import com.ruoyi.system.user.domain.vo.UserNameVO;

/**
 * 用户 业务层
 *
 * @author ruoyi
 */
public interface UserService
{
    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<UserVO> selectUserListScope(UserQuery user);

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<UserVO> selectAllocatedList(User user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<UserVO> selectUnallocatedList(User user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public UserVO selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public UserVO selectUserById(Long userId);

    /**
     * 根据用户ID查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    public String selectUserRoleGroup(String userName);

    /**
     * 根据用户ID查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
    public String selectUserPostGroup(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean checkPhoneUnique(User user);

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean checkEmailUnique(User user);

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    public void checkUserAllowed(User user);

    /**
     * 校验用户是否有数据权限
     *
     * @param userId 用户id
     */
    public void checkUserDataScope(Long userId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(UserVO user);

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public boolean registerUser(UserVO user);

    /**
     * 注册用户
     * @param phone 手机号
     * @param openId 微信openId
     * @param appId 应用ID
     * @return 结果
     */
    public UserVO registerUser(String phone, String openId, Long appId);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(UserVO user);

    /**
     * 用户授权角色
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    public void insertUserAuth(Long userId, Long[] roleIds);

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUserStatus(User user);

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUserProfile(User user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    public boolean updateUserAvatar(String userName, String avatar);

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    public int resetPwd(User user);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(String userName, String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    public int deleteUserByIds(Long[] userIds);

    /**
     * 根据用户名称列表查询用户
     *
     * @param nickNames
     * @return
     */
    List<UserVO> selectUserListByNickNames(List<String> nickNames);

    UserVO selectUserByPhonenumber(String phonenumber);

    /**
     * 查询用户列表
     */
    List<UserVO> selectUserList(UserQuery query);

    /**
     * 查询所有用户名称列表
     */
    List<UserNameVO> selectAllUserNameList();

    /**
     * 根据ID列表查询用户列表
     * @param ids
     * @return
     */
    List<UserVO> selectByIds(List<Long> ids);

    /**
     * 根据部门ID列表查询用户ID列表
     * @param deptIds
     * @return
     */
    List<Long> selectUserIdsByDeptIds(List<Long> deptIds);

    /**
     * 根据APP ID和openId查询用户
     * @param appId
     * @param openId
     * @return
     */
    public UserVO selectByAppOpenId(Long appId, String openId);

    /**
     * 增加爆灯次数
     * @param order 订单
     * @return 结果
     */
    public int addLightingNum(Order order);


    /**
     *
     * @param userId 用户ID
     * @param storeId 店铺ID
     * @param number 爆灯次数
     * @return
     */
    public int addLightingNum(Long userId, Long storeId, Integer number ,String bstType , Long bstId ,String reason);



    /**
     * 减少余额
     * @param userId 用户ID
     * @param amount 金额
     * @param bstType 业务类型
     * @param bstId 业务ID
     * @param reason 原因
     * @param checkBalance 是否检查余额是否充足
     * @return 结果
     */
    public int subtractBalance(Long userId, BigDecimal amount, BalanceLogBstType bstType, Long bstId, String reason, boolean checkBalance);

    /**
     * 更新用户运营区ID
     * @param userId 用户ID
     * @param areaId 运营区ID
     * @return 结果
     */
    public int updateAreaId(Long userId, Long areaId);


    /**
     * 查询单表数据（不允许联表）
     * @param userId 用户id
     * @return 用户信息
     */
    UserVO selectSimpleById(Long userId);

    /**
     * 根据店铺ID查询该店铺下所有下单的用户ID
     * @param query
     * @return
     */
    List<UserConsumeVO> selectUserIdsByStoreId(OrderQuery query);

    /**
     * 用户编辑个人基础信息
     * @param vo
     * @return
     */
    int updateUserBasicInfo(UserVO vo);
}
